#ifndef ELLIPTIC_ENERGY_HH
#define ELLIPTIC_ENERGY_HH

#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/common/stdstreams.hh>

#include <dune/fufem/interval.hh>
#include <dune/fufem/arithmetic.hh>

#include "localfriction.hh"

namespace Dune {
template <size_t dim> class EllipticEnergy {
public:
  using SmallVector = FieldVector<double, dim>;
  using SmallMatrix = FieldMatrix<double, dim, dim>;

  using NonlinearityType = LocalFriction<dim>;

  EllipticEnergy(SmallMatrix const &A, SmallVector const &b,
                 shared_ptr<NonlinearityType const> phi, size_t ignore = dim)
      : A(A), b(b), phi(phi), ignore(ignore) {}

  double operator()(SmallVector const &v) const {
    SmallVector y(0);
    Arithmetic::addProduct(y, 0.5, A, v);
    y -= b;
    return y * v + (*phi)(v);
  }

  SmallMatrix const &A;
  SmallVector const &b;
  shared_ptr<NonlinearityType const> const phi;
  /* Dimension that should be ignored; goes from 0 to dim-1; the
     special value dim means that no dimension should be ignored */
  size_t const ignore;

  void gradient(SmallVector const &x, SmallVector &y) const {
    A.mv(x, y);
    y -= b;
    phi->addGradient(x, y);

    if (ignore != dim)
      y[ignore] = 0;
  }
};
}
#endif
