#ifndef DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH
#define DUNE_TECTONIC_GLOBAL_RUINA_NONLINEARITY_HH

#include <vector>

#include <dune/common/bitsetvector.hh>
#include <dune/common/nullptr.hh>
#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/bvector.hh>

#include "globalnonlinearity.hh"
#include "localfriction.hh"
#include "frictionpotential.hh"

namespace Dune {
template <class MatrixType, class VectorType>
class GlobalRuinaNonlinearity
    : public GlobalNonlinearity<MatrixType, VectorType> {
public:
  using GlobalNonlinearity<MatrixType, VectorType>::block_size;
  using typename GlobalNonlinearity<MatrixType, VectorType>::FrictionType;

private:
  using typename GlobalNonlinearity<MatrixType,
                                    VectorType>::SingletonVectorType;

public:
  GlobalRuinaNonlinearity(Dune::BitSetVector<1> const &frictionalNodes,
                          SingletonVectorType const &nodalIntegrals,
                          FrictionData const &fd)
      : restrictions(nodalIntegrals.size()) {
    auto trivialNonlinearity =
        make_shared<FrictionType>(make_shared<TrivialFunction>());
    for (size_t i = 0; i < restrictions.size(); ++i) {
      restrictions[i] =
          frictionalNodes[i][0]
              ? make_shared<FrictionType>(
                    make_shared<FrictionPotential>(nodalIntegrals[i], fd))
              : trivialNonlinearity;
    }
  }

  void updateState(SingletonVectorType const &state) override {
    for (size_t i = 0; i < restrictions.size(); ++i)
      restrictions[i]->updateState(state[i]);
  }

  /*
    Return a restriction of the outer function to the i'th node.
  */
  shared_ptr<FrictionType> restriction(size_t i) const override {
    return restrictions[i];
  }

private:
  std::vector<shared_ptr<FrictionType>> restrictions;
};
}
#endif
