#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "friction_writer.hh"

double computeCOF(FrictionData const &fd, double V, double log_state) {
  double const mu = fd.mu0 + fd.a * std::log(V / fd.V0) +
                    fd.b * (log_state + std::log(fd.V0 / fd.L));
  return (mu > 0) ? mu : 0;
}

template <class BitVectorType>
FrictionWriter<BitVectorType>::FrictionWriter(
    FrictionData const &_fd, BitVectorType const &_boundaryNodes)
    : coefficientWriter("coefficients", std::fstream::out),
      displacementWriter("displacements", std::fstream::out),
      stateWriter("states", std::fstream::out),
      velocityWriter("velocities", std::fstream::out),
      fd(_fd),
      boundaryNodes(_boundaryNodes) {}

template <class BitVectorType>
FrictionWriter<BitVectorType>::~FrictionWriter() {
  stateWriter.close();
  displacementWriter.close();
  velocityWriter.close();
  coefficientWriter.close();
}

template <class BitVectorType>
template <class SingletonVectorType, class VectorType>
void FrictionWriter<BitVectorType>::writeInfo(SingletonVectorType const &alpha,
                                              VectorType const &u,
                                              VectorType const &v) {
  for (size_t i = 0; i < boundaryNodes.size(); ++i) {
    if (!boundaryNodes[i][0])
      continue;

    coefficientWriter << computeCOF(fd, v[i].two_norm(), alpha[i]) << " ";
    displacementWriter << u[i][0] << " ";
    stateWriter << alpha[i][0] << " ";
    velocityWriter << v[i][0] << " ";
  }

  stateWriter << std::endl;
  displacementWriter << std::endl;
  velocityWriter << std::endl;
  coefficientWriter << std::endl;
}

#include "friction_writer_tmpl.cc"
